@@ -969,6 +969,19 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 
       break;
 
+    case GST_MAKE_FOURCC ('L', 'A', 'G', 'S'):
+      caps = gst_caps_new_empty_simple ("video/x-lagarith");
+      if (codec_name)
+        *codec_name = g_strdup ("Lagarith lossless video codec");
+      break;
+
+    case GST_MAKE_FOURCC ('M', '1', '0', '1'):
+    case GST_MAKE_FOURCC ('M', '1', '0', '2'):
+      caps = gst_caps_new_empty_simple ("video/x-m101");
+      if (codec_name)
+        *codec_name = g_strdup ("Matrox uncompressed SD video codec");
+      break;
+
     default:
       GST_WARNING ("Unknown video fourcc %" GST_FOURCC_FORMAT,
           GST_FOURCC_ARGS (codec_fcc));
@@ -1138,72 +1151,36 @@ static gboolean
 gst_riff_wave_add_default_channel_mask (GstCaps * caps,
     gint nchannels, gint channel_reorder_map[18])
 {
-  guint64 channel_mask = 0;
+  if (nchannels <= 1)
+    return TRUE;
+
+  guint64 channel_mask = gst_audio_channel_get_fallback_mask (nchannels);
   static const gint reorder_maps[8][11] = {
     {0,},
     {0, 1},
-    {-1, -1, -1},
+    {0, 1, 2},
     {0, 1, 2, 3},
     {0, 1, 3, 4, 2},
     {0, 1, 4, 5, 2, 3},
-    {-1, -1, -1, -1, -1, -1, -1},
+    {0, 1, 4, 5, 2, 3, 6},
     {0, 1, 4, 5, 2, 3, 6, 7}
   };
 
-  if (nchannels > 8) {
-    GST_DEBUG ("invalid number of channels: %d", nchannels);
-    return FALSE;
-  }
-
-  /* This uses the default channel mapping from ALSA which
-   * is used in quite a few surround test files and seems to be
-   * the defacto standard. The channel mapping from
-   * WAVE_FORMAT_EXTENSIBLE doesn't seem to be used in normal
-   * wav files like chan-id.wav.
-   * http://bugzilla.gnome.org/show_bug.cgi?id=489010
-   */
-  switch (nchannels) {
-    case 1:
-      /* Mono => nothing */
-      if (channel_reorder_map)
-        channel_reorder_map[0] = 0;
-      return TRUE;
-    case 8:
-      channel_mask |=
-          G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT;
-      channel_mask |=
-          G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT;
-      /* fall through */
-    case 6:
-      channel_mask |= G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_LFE1;
-      /* fall through */
-    case 5:
-      channel_mask |=
-          G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
-      /* fall through */
-    case 4:
-      channel_mask |=
-          G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
-      channel_mask |=
-          G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_REAR_LEFT;
-      /* fall through */
-    case 2:
-      channel_mask |=
-          G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
-      channel_mask |=
-          G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
-      break;
-    default:
-      return FALSE;
-  }
-
-  if (channel_reorder_map)
+  if (channel_reorder_map && nchannels <= 8) {
     memcpy (channel_reorder_map, reorder_maps[nchannels - 1],
         sizeof (gint) * nchannels);
+  }
 
   gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
       NULL);
 
+  if (channel_mask == 0) {
+    GST_WARNING
+        ("Failed to set default channel mask for %d channels - marking as 'unposition",
+        nchannels);
+    return FALSE;
+  }
+
   return TRUE;
 }
 
@@ -1219,18 +1196,24 @@ gst_riff_wavext_get_default_channel_mask (guint nchannels)
     case 11:
       channel_mask |= 0x00400;
       channel_mask |= 0x00200;
+      /* FALLTHROUGH */
     case 9:
       channel_mask |= 0x00100;
+      /* FALLTHROUGH */
     case 8:
       channel_mask |= 0x00080;
       channel_mask |= 0x00040;
+      /* FALLTHROUGH */
     case 6:
       channel_mask |= 0x00020;
       channel_mask |= 0x00010;
+      /* FALLTHROUGH */
     case 4:
       channel_mask |= 0x00008;
+      /* FALLTHROUGH */
     case 3:
       channel_mask |= 0x00004;
+      /* FALLTHROUGH */
     case 2:
       channel_mask |= 0x00002;
       channel_mask |= 0x00001;
@@ -1266,7 +1249,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
           GST_WARNING ("Invalid block align: %d > %d", ba, (32 / 8) * ch);
           wd = GST_ROUND_UP_8 (strf->bits_per_sample);
         } else if (ba != 0) {
-          /* If we have an empty blockalign, we take the width contained in 
+          /* If we have an empty blockalign, we take the width contained in
            * strf->bits_per_sample */
           wd = ba * 8 / ch;
         } else {
@@ -1323,8 +1306,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
          * so either we calculate the bitrate or mark it as invalid as this
          * would probably confuse timing */
         strf->av_bps = 0;
-        if (strf->channels != 0 && strf->rate != 0 && strf->blockalign != 0) {
-          int spb = ((strf->blockalign - strf->channels * 7) / 2) * 2;
+        if (strf->channels != 0 && strf->rate != 0 && strf->blockalign != 0 &&
+            (strf->blockalign / strf->channels) >= 7) {
+          int spb = ((strf->blockalign / strf->channels) - 7) * 2 + 2;
           strf->av_bps =
               gst_util_uint64_scale_int (strf->rate, strf->blockalign, spb);
           GST_DEBUG ("fixing av_bps to calculated value %d of MS ADPCM",
@@ -1445,8 +1429,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
          * header, so either we calculate the bitrate or mark it as invalid
          * as this would probably confuse timing */
         strf->av_bps = 0;
-        if (strf->channels != 0 && strf->rate != 0 && strf->blockalign != 0) {
-          int spb = ((strf->blockalign - strf->channels * 4) / 2) * 2;
+        if (strf->channels != 0 && strf->rate != 0 && strf->blockalign != 0 &&
+            (strf->blockalign / strf->channels) >= 4) {
+          int spb = ((strf->blockalign / strf->channels) - 4) * 2 + 1;
           strf->av_bps =
               gst_util_uint64_scale_int (strf->rate, strf->blockalign, spb);
           GST_DEBUG ("fixing av_bps to calculated value %d of IMA DVI ADPCM",
@@ -1773,7 +1758,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
             GST_RIFF_WAVE_FORMAT_EXTENSIBLE) {
           GST_DEBUG ("WAVE_FORMAT_EXTENSIBLE nested");
         } else {
-          /* recurse where no special consideration has yet to be identified 
+          /* recurse where no special consideration has yet to be identified
            * for the subformat guid */
           caps = gst_riff_create_audio_caps (subformat_guid[0], strh, strf,
               strf_data, strd_data, codec_name, channel_reorder_map);
